@@ -9,30 +9,24 @@ import PyQt5.QtCore as QtCore
 import logging
 import sys
 import argparse
+
+import abstract_instrument_interface
 from pyThorlabsPM100x.driver import ThorlabsPM100x
 from pyThorlabsPM100x.plots import PlotObject
-from pyThorlabsPM100x.abstract_interface import abstract_interface, abstract_gui
 
 graphics_dir = os.path.join(os.path.dirname(__file__), 'graphics')
 
-class interface(abstract_interface):
+class interface(abstract_instrument_interface.abstract_interface):
     """
-    Create a high-level interface with the device, and provide a GUI
-
+    Create a high-level interface with the device, and act as a connection between the low-level
+    interface (i.e. the driver) and the gui.
+    Several general-purpose attributes and methods are defined in the class abstract_interface defined in abstract_instrument_interface
     ...
 
-    Attributes
+    Attributes specific for this class (see the abstract class abstract_instrument_interface.abstract_interface for general attributes)
     ----------
-    output : dict
-        a dictionary containing the output data created by this interface
-    app : Qt.QApplication
-        The pyqt5 QApplication() object
-    mainwindow  : Qt.QWidget
-        The main window of the Qt application
-    instrument : ThorlabsPM100x
-        a driver object, for the low-level communication with the device
-    gui : gui()
-        instance of the class gui(), containing all guis element and basic function to set different states
+    instrument
+        Instance of driver.ThorlabsPM100x
     connected_device_name : str
         Name of the physical device currently connected to this interface 
     continuous_read : bool 
@@ -45,26 +39,21 @@ class interface(abstract_interface):
         The power units of this device
     current_power_string : str 
         Last power read from powermeter, as a string
-    verbose : bool
-        Sets wheter the logger of this interface will be verbose or not (default = True)
-    name_logger : str
-        Name of the logger of this interface (default = __package__). The logger formatter will be "[name_logger]:...".
 
 
-    Methods
+    Methods defined in this class (see the abstract class abstract_instrument_interface.abstract_interface for general methods)
     -------
+    refresh_list_devices()
+        Get a list of compatible devices from the driver. Stores them in self.list_devices, and populates the combobox in the GUI.
     connect_device(device_full_name)
         Connect to the device identified by device_full_name
-    disconnect_device(device_full_name)
+    disconnect_device()
         Disconnect the currently connected device
     close()
-        Closes this interface, disconnect device (if any) and close plot window
-    create_gui(parent)
-    
-    refresh_list_devices()
+        Closes this interface, close plot window (if any was open), and calls the close() method of the parent class, which typically calls the disconnect_device method
     
     set_disconnected_state()
-    
+        
     set_connecting_state()
     
     set_connected_state()
@@ -77,9 +66,9 @@ class interface(abstract_interface):
     
     change_power_range(direction)
     
-    set_power_range_auto(status)
+    set_auto_power_range(status)
     
-    read_status_power_autorange()
+    read_auto_power_range()
     
     set_zero_powermeter()
     
@@ -90,8 +79,6 @@ class interface(abstract_interface):
     stop_reading()
     
     update()
-    
-    set_trigger(external_function,delay=0)
 
     """
 
@@ -99,62 +86,62 @@ class interface(abstract_interface):
 
     def __init__(self, **kwargs):
         self.output = {'Power':0} 
-        self.continuous_read = False #When this is set to True, the data from device are acquired continuosly at the rate set by self.refresh_time
-        self.refresh_time = 0.2 #default refresh rate, in seconds
+        
+        ### Default values of settings (might be overlapped by settings saved in .json files later)
+        self.settings = {   'refresh_time': 0.2,
+                            'auto_power_range': True
+                            }
+        
+        self.list_devices = []          #list of devices found   
+        self.continuous_read = False    # When this is set to True, the data from device are acquired continuosly at the rate set by self.refresh_time
         self.stored_data = [] # List used to store data acquired by device
         self.current_power_string = " " # Last power read from powermeter, as a string
         self.connected_device_name = ''
-
+        ###
         self.instrument = ThorlabsPM100x() 
+        ###
+        self.gui_class = gui
+        ###
         super().__init__(**kwargs)
- 
-    def create_gui(self,parent,plot=False): 
-        '''
-            parent, QWidget        
-                a QWidget (or QMainWindow) object that will be the parent for the gui of this device.
-                When this script is ran on its own (i.e. not within another gui) self.mainwindow and parent are actually the same object
-            plot, boolean
-                If set true, the GUI also generates a plot object (and a button to show/hide the plot) to plot the content of the self.stored_data object
-        '''
-        self.gui = gui(self,parent,plot)
-        self.gui.initialize() 
 
-### Functions to talk with low-level driver
+############################################################
+### Functions to interface the GUI and low-level driver
+############################################################
 
     def refresh_list_devices(self):
         '''
         Get a list of all devices connected, by using the method list_devices() of the driver. For each device obtain its identity and its address.
         For each device, create the string "identity -->  address" and add the string to the corresponding combobox in the GUI 
         '''
-        
-        #First we empty the combobox
-        self.gui.combo_Devices.clear()
-        
-        #Then we read the list of devices
+        self.gui.combo_Devices.clear()                      #First we empty the combobox       
+        self.list_devices = []
         self.logger.info(f"Looking for devices...") 
-        list_valid_devices = self.instrument.list_devices()
+        list_valid_devices = self.instrument.list_devices() #Then we read the list of devices
+        self.list_devices = list_valid_devices
         if(len(list_valid_devices)>0):
             list_IDNs_and_devices = [dev[1] + " --> " + dev[0] for dev in list_valid_devices] 
             self.gui.combo_Devices.addItems(list_IDNs_and_devices)  
         self.logger.info(f"Found {len(list_valid_devices)} devices.") 
 
     def connect_device(self,device_full_name):
-        #device_full_name = self.combo_Devices.currentText() # Get the device name from the combobox
-        if(device_full_name==''): # Check  that the name is not empty
-            self.logger.error("No valid device has been selected")
+        if(device_full_name==''): 
+            self.logger.error("No valid device has been selected.")
             return
         self.set_connecting_state()
-        device_name = device_full_name.split(' --> ')[1].lstrip() # We extract the device address from the device name
+        device_name = device_full_name.split(' --> ')[1].lstrip()   # We extract the device address from the device name
         self.logger.info(f"Connecting to device {device_name}...")
-        (Msg,ID) = self.instrument.connect_device(device_name) # Try to connect by using the method ConnectDevice of the powermeter object
-        if(ID==1):  #If connection was successful
-            self.logger.info(f"Connected to device {device_name}.")
-            self.connected_device_name = device_name
-            self.power_units = self.instrument.power_units
-            self.set_connected_state()
-            self.start_reading()
-        else: #If connection was not successful
-            self.logger.error(f"Error: {Msg}")
+        try:
+            (Msg,ID) = self.instrument.connect_device(device_name)      # Try to connect by using the method ConnectDevice of the powermeter object
+            if(ID==1):  #If connection was successful
+                self.logger.info(f"Connected to device {device_name}.")
+                self.connected_device_name = device_name
+                self.set_connected_state()
+                self.start_reading()
+            else: #If connection was not successful
+                self.logger.error(f"Error: {Msg}")
+                self.set_disconnected_state()
+        except Exception as e:
+            self.logger.error(f"Error: {e}")
             self.set_disconnected_state()
 
     def disconnect_device(self):
@@ -167,8 +154,13 @@ class interface(abstract_interface):
         else: #If disconnection was not successful
             self.logger.error(f"Error: {Msg}")
             self.set_disconnected_state() #When disconnection is not succeful, it is typically because the device alredy lost connection
-                                          #for some reason. In this case, it is still useful to have the widget reset to disconnected state      
-            
+                                          #for some reason. In this case, it is still useful to have all widgets reset to disconnected state      
+    def close(self,**kwargs):
+        if hasattr(self.gui,'plot_window'):
+            if self.gui.plot_window:
+                self.gui.plot_window.close()
+        super().close(**kwargs)           
+    
     def set_disconnected_state(self):
         self.gui.set_disconnected_state()
 
@@ -176,28 +168,29 @@ class interface(abstract_interface):
         self.gui.set_connecting_state()
 
     def set_connected_state(self):
+        self.power_units = self.instrument.power_units
         self.gui.set_connected_state()
         self.read_wavelength()
-        self.read_status_power_autorange()
+        #self.read_status_power_autorange()
+        self.set_auto_power_range(self.settings['auto_power_range'])
         self.read_power_range()
-  
 
     def set_refresh_time(self, refresh_time):
         try: 
             refresh_time = float(refresh_time)
-            if self.refresh_time == refresh_time: #in this case the number in the refresh time edit box is the same as the refresh time currently stored
+            if self.settings['refresh_time'] == refresh_time: #in this case the number in the refresh time edit box is the same as the refresh time currently stored
                 return True
         except ValueError:
             self.logger.error(f"The refresh time must be a valid number.")
-            self.gui.edit_RefreshTime.setText(f"{self.refresh_time:.3f}")
+            self.gui.edit_RefreshTime.setText(f"{self.settings['refresh_time']:.3f}")
             return False
         if refresh_time < 0.001:
             self.logger.error(f"The refresh time must be positive and >= 1ms.")
-            self.gui.edit_RefreshTime.setText(f"{self.refresh_time:.3f}")
+            self.gui.edit_RefreshTime.setText(f"{self.settings['refresh_time']:.3f}")
             return False
         self.logger.info(f"The refresh time is now {refresh_time} s.")
-        self.refresh_time = refresh_time
-        self.gui.edit_RefreshTime.setText(f"{self.refresh_time:.3f}")
+        self.settings['refresh_time'] = refresh_time
+        self.gui.edit_RefreshTime.setText(f"{self.settings['refresh_time']:.3f}")
         return True
 
     def set_wavelength(self, wl):
@@ -206,7 +199,7 @@ class interface(abstract_interface):
                     return True
             self.logger.info(f"Setting the wavelength to {wl} for the device {self.connected_device_name}...")
         except ValueError:
-            self.logger.error(f"The refresh time must be a valid number.")
+            self.logger.error(f"The wavelength must be a valid number.")
             self.gui.edit_Wavelength.setText(str(self.instrument.wavelength))
             return False
         try: 
@@ -249,30 +242,32 @@ class interface(abstract_interface):
         self.gui.edit_PowerRange.setCursorPosition(1)
         self.logger.info(f"Current power range is {self.power_range}.") 
 
-    def set_power_range_auto(self,status):
+    def set_auto_power_range(self,status):
+        status = bool(status)
         status_string = 'ON' if status else 'OFF'
         self.logger.info(f"Setting the auto-ranging function to {status_string} for the device {self.connected_device_name}...")    
         try:
             self.instrument.auto_power_range  = status
+            self.gui.set_auto_power_range_state(status)
             self.logger.info(f"Setting changed succesfully.")
+            self.settings['auto_power_range'] = status
+            self.gui.box_PowerRangeAuto.setChecked(status)
         except Exception as e:
             self.logger.error(f"An error occurred while setting the auto-ranging status: {e}")
 
-        self.read_status_power_autorange()
+        #self.read_auto_power_range()
         self.read_power_range()
 
-    def read_status_power_autorange(self):
+    def read_auto_power_range(self):
         self.logger.info(f"Reading the status of the auto-ranging function for the device {self.connected_device_name}...")   
         try:
             auto_power_range = self.instrument.auto_power_range
+            status_string = 'ON' if auto_power_range else 'OFF'
             self.gui.box_PowerRangeAuto.setChecked(auto_power_range)
-            if auto_power_range:
-                status = 'ON'
-                self.gui.disable_widget([self.gui.edit_PowerRange,self.gui.button_IncreasePowerRange, self.gui.button_DecreasePowerRange])
-            else:
-                status = 'OFF'
-                self.gui.enable_widget([self.gui.edit_PowerRange,self.gui.button_IncreasePowerRange, self.gui.button_DecreasePowerRange])
-            self.logger.info(f"The auto-ranging function is currently set to {status}.")
+            self.gui.set_auto_power_range_state(status)
+            self.logger.info(f"The auto-ranging function is currently set to {status_string}.")
+            self.settings['auto_power_range'] = auto_power_range
+            return auto_power_range
         except Exception as e:
             self.logger.error(f"An error occurred while reading the auto-ranging status: {e}")
 
@@ -302,22 +297,22 @@ class interface(abstract_interface):
 
         self.continuous_read = True #Until this variable is set to True, the function UpdatePower will be repeated continuosly 
         self.logger.info(f"Starting reading from device {self.connected_device_name}...")
-        # Call the function self.update(), which will do stome suff (read power and store it in a global variable) and then call itself continuosly until the variable self.ContinuousRead is set back to 0
+        # Call the function self.update(), which will do stome suff (read power and store it in a global variable) and then call itself continuosly until the variable self.continuous_read is set to False
         self.update()
         return
  
     def pause_reading(self):
-        #Sets self.ContinuousRead to 0 (this will force the function Update() to stop calling itself)
+        #Sets self.continuous_read to False (this will force the function update() to stop calling itself)
         self.continuous_read = False
         self.logger.info(f"Paused reading from device {self.connected_device_name}.")
         self.gui.set_pause_state() # Change some widgets
         return
 
     def stop_reading(self):
-        #Sets self.ContinuousRead to 0 (this will force the function Update() to stop calling itself)
+        #Sets self.continuous_read to False (this will force the function update() to stop calling itself) and delete all accumulated data
         self.continuous_read = False
         self.stored_data = []
-        self.update() #We call one more time the self.Update() function to make sure plots is cleared. Since self.continuous_read is already set to False, Update() will not acquire data anymore
+        self.update() #We call one more time the self.update() function to make sure plots is cleared. Since self.continuous_read is already set to False, update() will not acquire data anymore
         self.logger.info(f"Stopped reading from device {self.connected_device_name}. All stored data have been deleted.")
         self.gui.set_stopped_state() # Change some widgets
         # ...
@@ -344,60 +339,79 @@ class interface(abstract_interface):
 
             self.current_power_string = f"{currentPower:.2e}" + ' ' +  power_units
             self.gui.edit_Power.setText(self.current_power_string)
-            QtCore.QTimer.singleShot(int(self.refresh_time*1e3), self.update)
+            QtCore.QTimer.singleShot(int(self.settings['refresh_time']*1e3), self.update)
            
         return
 
-    ### END Functions to talk with low-level driver
+############################################################
+### END Functions to interface the GUI and low-level driver
+############################################################
     
     
-class gui(abstract_gui):
+class gui(abstract_instrument_interface.abstract_gui):
+     
     def __init__(self,interface,parent,plot=False):
+        """
+        Attributes specific for this class (see the abstract class abstract_instrument_interface.abstract_gui for general attributes)
+        ----------
+        plot, bool
+            If set true, the GUI also generates a plot object (and a button to show/hide the plot) to plot the content of the self.stored_data object
+        """
         super().__init__(interface,parent)
 
         self.widgets_enabled_when_connected = []     #The widgets in this list will only be enabled when the interface has succesfully connected to a device
         self.widgets_enabled_when_disconnected = []  #The widgets in this list will only be enabled when the interface is not connected to a device
         self.plot_window = None # QWidget object of the widget (i.e. floating window) that will contain the plot
         self.plot_object = None # PlotObject object of the plot where self.store_powers is plotted
-        
+
+        if plot:        # Create a plot object
+            self.create_plot() 
+
+    def initialize(self):
+        self.create_widgets()
+
+        ### SET INITIAL STATE OF WIDGETS
+        self.edit_Power.setText(self.interface.current_power_string)
+        self.box_PowerRangeAuto.setChecked(self.interface.settings['auto_power_range'])
+        self.edit_RefreshTime.setText(f"{self.interface.settings['refresh_time']:.3f}")
+        self.interface.refresh_list_devices()    #By calling this method, as soon as the gui is created we also look for devices
+        self.set_disconnected_state()               #When GUI is created, all widgets are set to the "Disconnected" state
+        ###
+
+        self.connect_widgets_events_to_functions()
+
+        ### Call the initialize method of the parent class
+        super().initialize()
+
+    def create_widgets(self):
         hbox1 = Qt.QHBoxLayout()
         self.label_DeviceList = Qt.QLabel("Devices: ")
         self.combo_Devices = Qt.QComboBox()
-        #self.combo_Devices.resize(self.combo_Devices.sizeHint())
         self.button_RefreshDeviceList = Qt.QPushButton("")
-        self.button_RefreshDeviceList.setIcon(QtGui.QIcon(os.path.join(graphics_dir,'refresh.png')))
-        self.button_RefreshDeviceList.clicked.connect(self.click_button_refresh_list_devices)
+        self.button_RefreshDeviceList.setIcon(QtGui.QIcon(os.path.join(graphics_dir,'refresh.png')))     
         hbox1.addWidget(self.label_DeviceList)
         hbox1.addWidget(self.combo_Devices,stretch=1)
         hbox1.addWidget(self.button_RefreshDeviceList)
-        #hbox1.addStretch(1)
 
         hbox2 = Qt.QHBoxLayout()
         self.button_ConnectDevice = Qt.QPushButton("Connect")
-        self.button_ConnectDevice.clicked.connect(self.click_button_connect_disconnect)
-        self.button_SetZeroPowermeter = Qt.QPushButton("Set Zero")
-        self.button_SetZeroPowermeter.clicked.connect(self.click_button_set_zero_powermeter)
+        self.button_SetZeroPowermeter = Qt.QPushButton("Set Zero")    
         self.label_Wavelength = Qt.QLabel("Wavelength: ")
         self.edit_Wavelength = Qt.QLineEdit()
-        self.edit_Wavelength.returnPressed.connect(self.press_enter_wavelength)
         self.edit_Wavelength.setAlignment(QtCore.Qt.AlignRight)
-        #self.edit_Wavelength.setMaximumWidth(50)
         self.label_WavelengthUnits = Qt.QLabel("nm")
         self.label_PowerRange = Qt.QLabel("Power range: ")
         self.button_DecreasePowerRange = Qt.QPushButton("<")
         self.button_DecreasePowerRange.setToolTip('Decrease the powermeter power range.')
-        self.button_DecreasePowerRange.setMaximumWidth(15)
-        self.button_DecreasePowerRange.clicked.connect(lambda x:self.click_button_change_power_range(-1))
+        self.button_DecreasePowerRange.setMaximumWidth(15)       
         self.edit_PowerRange = Qt.QLineEdit()
         self.edit_PowerRange.setToolTip('Maximum power measurable in the current power range (unless \'Auto\' is checked).')
-        #self.edit_PowerRange.setMaximumWidth(60)
         self.edit_PowerRange.setReadOnly(True)
         self.button_IncreasePowerRange = Qt.QPushButton(">")
         self.button_IncreasePowerRange.setToolTip('Increase the powermeter power range.')
         self.button_IncreasePowerRange.setMaximumWidth(15)
-        self.button_IncreasePowerRange.clicked.connect(lambda x:self.click_button_change_power_range(+1))
         self.box_PowerRangeAuto = Qt.QCheckBox("Auto")
-        self.box_PowerRangeAuto.stateChanged.connect(self.click_box_PowerRangeAuto)
+
         self.box_PowerRangeAuto.setToolTip('Set the power range of the powermeter to Automatic.')
         hbox2.addWidget(self.button_ConnectDevice)
         hbox2.addWidget(self.button_SetZeroPowermeter)
@@ -409,53 +423,41 @@ class gui(abstract_gui):
         hbox2.addWidget(self.edit_PowerRange)
         hbox2.addWidget(self.button_IncreasePowerRange)
         hbox2.addWidget(self.box_PowerRangeAuto)
-        #hbox2.addStretch(1)
 
         hbox3 = Qt.QHBoxLayout()
         self.button_StartPauseReading = Qt.QPushButton("")
         self.button_StartPauseReading.setIcon(QtGui.QIcon(os.path.join(graphics_dir,'play.png')))
         self.button_StartPauseReading.setToolTip('Start or pause the reading from the powermeter. The previous data points are not discarded when pausing.') 
-        self.button_StartPauseReading.clicked.connect(self.click_button_StartPauseReading)
         self.button_StopReading = Qt.QPushButton("")
         self.button_StopReading.setIcon(QtGui.QIcon(os.path.join(graphics_dir,'stop.png')))
         self.button_StopReading.setToolTip('Stop the reading from the powermeter. All previous data points are discarded.') 
-        self.button_StopReading.clicked.connect(self.click_button_StopReading)
         self.label_RefreshTime = Qt.QLabel("Refresh time (s): ")
         self.label_RefreshTime.setToolTip('Specifies how often the power is read from the powermeter (Minimum value = 0.001 s).') 
         self.edit_RefreshTime  = Qt.QLineEdit()
-        self.edit_RefreshTime.setText(f"{self.interface.refresh_time:.3f}")
         self.edit_RefreshTime.setToolTip('Specifies how often the power is read from the powermeter (Minimum value = 0.001 s).') 
-        self.edit_RefreshTime.returnPressed.connect(self.press_enter_refresh_time)
         self.edit_RefreshTime.setAlignment(QtCore.Qt.AlignRight)
-        #self.edit_RefreshTime.setMaximumWidth(50)
-
         font = QtGui.QFont("Times", 12,QtGui.QFont.Bold)
         self.label_Power = Qt.QLabel("Power: ")
         self.label_Power.setAlignment(QtCore.Qt.AlignRight | QtCore.Qt.AlignCenter)
         self.label_Power.setFont(font)
         self.edit_Power = Qt.QLineEdit()
         self.edit_Power.setFont(font)
-        self.edit_Power.setText(self.interface.current_power_string)
-        #self.edit_Power.setMaximumWidth(150)
-        #self.edit_Power.resize(self.edit_Power.sizeHint());
         self.edit_Power.setAlignment(QtCore.Qt.AlignRight)
         self.edit_Power.setReadOnly(True)
-        if plot:
-            self.button_ShowHidePlot = Qt.QPushButton("Show/Hide Plot")
-            #self.button_StopReading.setIcon(QtGui.QIcon(os.path.join(graphics_dir,'stop.png')))
-            self.button_ShowHidePlot.setToolTip('Show/Hide Plot.') 
-            self.button_ShowHidePlot.clicked.connect(self.click_button_ShowHidePlot)
-
         hbox3.addWidget(self.button_StartPauseReading)
-        if plot:
+
+        if self.plot_object:
+            self.button_ShowHidePlot = Qt.QPushButton("Show/Hide Plot")
+            self.button_ShowHidePlot.setToolTip('Show/Hide Plot.')
             hbox3.addWidget(self.button_StopReading)
+
         hbox3.addWidget(self.label_RefreshTime)
         hbox3.addWidget(self.edit_RefreshTime)
         hbox3.addWidget(self.label_Power)
         hbox3.addWidget(self.edit_Power)
-        if plot:
+
+        if self.plot_object:
             hbox3.addWidget(self.button_ShowHidePlot)
-        #hbox3.addStretch(1)
                 
         self.container = Qt.QVBoxLayout()
         self.container.addLayout(hbox1)  
@@ -473,27 +475,22 @@ class gui(abstract_gui):
                                                self.button_StopReading]
         self.widgets_enabled_when_disconnected = [self.combo_Devices , 
                                                   self.button_RefreshDeviceList]
-        if plot:
-            self.create_plot()
-    
-    def initialize(self):
-        super().initialize()
-        self.click_button_refresh_list_devices()    #By calling this method, as soon as the gui is created we also look for devices
-        self.set_disconnected_state()               #When GUI is created, all widgets are set to the "Disconnected" state
-        
-    def create_plot(self):
-        '''
-        This function creates an additional (separated) window with a pyqtgraph object, which plots the contents of self.stored_data
-        '''
-        self.plot_window = Qt.QWidget() #This is the widget that will contain the plot. Since it does not have a parent, the plot will be in a floating (separated) window
-        self.plot_object = PlotObject(self.interface.app, self.interface.mainwindow, self.plot_window)
-        styles = {"color": "#fff", "font-size": "20px"}
-        self.plot_object.graphWidget.setLabel("left", "Power", **styles)
-        self.plot_object.graphWidget.setLabel("bottom", "Acqusition #", **styles)
-        self.plot_window.setWindowTitle(f"Powermeter: {self.interface.connected_device_name}")
-        self.plot_window.show()
-        self.plot_window.setHidden(True)
-            
+
+    def connect_widgets_events_to_functions(self):
+        self.button_RefreshDeviceList.clicked.connect(self.click_button_refresh_list_devices)
+        self.button_ConnectDevice.clicked.connect(self.click_button_connect_disconnect)
+        self.button_SetZeroPowermeter.clicked.connect(self.click_button_set_zero_powermeter)
+        self.edit_Wavelength.returnPressed.connect(self.press_enter_wavelength)
+        self.button_DecreasePowerRange.clicked.connect(lambda x:self.click_button_change_power_range(-1))
+        self.button_IncreasePowerRange.clicked.connect(lambda x:self.click_button_change_power_range(+1))
+        self.box_PowerRangeAuto.stateChanged.connect(self.click_box_PowerRangeAuto)
+        self.button_StartPauseReading.clicked.connect(self.click_button_StartPauseReading)
+        self.button_StopReading.clicked.connect(self.click_button_StopReading)
+        self.edit_RefreshTime.returnPressed.connect(self.press_enter_refresh_time)
+
+        if self.plot_object:
+            self.button_ShowHidePlot.clicked.connect(self.click_button_ShowHidePlot)
+
     def set_disconnected_state(self):
         self.disable_widget(self.widgets_enabled_when_connected)
         self.enable_widget(self.widgets_enabled_when_disconnected)
@@ -523,7 +520,15 @@ class gui(abstract_gui):
     def set_stopped_state(self):    
         self.button_StartPauseReading.setIcon(QtGui.QIcon(os.path.join(graphics_dir,'play.png')))
 
+    def set_auto_power_range_state(self,auto_power_range):
+        if auto_power_range:
+            self.disable_widget([self.edit_PowerRange,self.button_IncreasePowerRange, self.button_DecreasePowerRange])
+        else:
+            self.enable_widget([self.edit_PowerRange,self.button_IncreasePowerRange, self.button_DecreasePowerRange])
+            
+############################################################
 ### GUI Events Functions
+############################################################
 
     def click_button_refresh_list_devices(self):
         self.interface.refresh_list_devices()
@@ -540,7 +545,7 @@ class gui(abstract_gui):
             status_bool = True
         else:
             status_bool = False
-        self.interface.set_power_range_auto(status_bool)
+        self.interface.set_auto_power_range(status_bool)
 
     def click_button_set_zero_powermeter(self):
         self.interface.set_zero_powermeter()
@@ -566,7 +571,25 @@ class gui(abstract_gui):
 
     def click_button_ShowHidePlot(self):
         self.plot_window.setHidden(not self.plot_window.isHidden())
-  
+
+############################################################
+### END GUI Events Functions
+############################################################
+
+    def create_plot(self):
+        '''
+        This function creates an additional (separated) window with a pyqtgraph object, which plots the contents of self.stored_data
+        '''
+        self.plot_window = Qt.QWidget() #This is the widget that will contain the plot. Since it does not have a parent, the plot will be in a floating (separated) window
+        self.plot_object = PlotObject(self.interface.app, self.interface.mainwindow, self.plot_window)
+        styles = {"color": "#fff", "font-size": "20px"}
+        self.plot_object.graphWidget.setLabel("left", "Power", **styles)
+        self.plot_object.graphWidget.setLabel("bottom", "Acqusition #", **styles)
+        self.plot_window.setWindowTitle(f"Powermeter: {self.interface.connected_device_name}")
+        self.plot_window.show()
+        self.plot_window.setHidden(True)
+            
+
 
 class MainWindow(Qt.QWidget):
     def __init__(self):
@@ -575,7 +598,9 @@ class MainWindow(Qt.QWidget):
 
     def closeEvent(self, event):
         if self.child:
-            self.child.close()
+            pass#self.child.close()
+
+#################################################################################################
 
 def main():
     parser = argparse.ArgumentParser(description = "",epilog = "")
@@ -588,10 +613,10 @@ def main():
     Interface.verbose = not(args.decrease_verbose)
     app.aboutToQuit.connect(Interface.close) 
     Interface.create_gui(window,plot=True)
-    #Interface.set_trigger(lambda : print('test'),delay=1000)
-    
     window.show()
     app.exec()# Start the event loop.
 
 if __name__ == '__main__':
     main()
+
+#################################################################################################
